package Mythtv::Synchronise;

use strict;
use Carp;
use DBI;
use File::Find;
use lib 'Configuration';

use Parameters ();              # Application wide configuration
use vars qw(%items);            # Grab the datastructure from the config file
*items = \%Parameters::items;   # Alias the variable out

my %sql = (
            vidtypes => qq/select extension from videotypes where f_ignore = 0/,
            vidloc   => qq/select data from settings where value = 'VideoStartupDir'/,
            mythvids => qq/select filename from videometadata/,
            addvid   => qq/insert into videometadata ( title, director, plot, rating, year, userrating, filename, coverfile, showlevel) 
                           values (?,'Unknown','None','NR',1970,0,?, 'No Cover', 1)/,
            rmvid    => qq/delete from videometadata where filename = ?/,
          );

{

    my @FSfiles; #global

    sub _getDBH {
        my $self = shift;

        my $host     = $self->{config}{videometadata}{database}{host};
        my $database = $self->{config}{videometadata}{database}{name};
        my $user     = $self->{config}{videometadata}{database}{user};
        my $password = $self->{config}{videometadata}{database}{password};

        $self->{_dbh} = DBI->connect("DBI:mysql:$database:$host", $user, $password) || croak "Unable to get DB handle!";

    }

    # Generic update/insert method
    sub _addVid {
        my $self = shift;
        my %args = @_;

        $self->_getDBH if (! defined $self->{_dbh});
        my $sth= $self->{_dbh}->prepare( $sql{addvid} );

        $sth->execute($args{title}, $args{filename});

    }

    # Generic update/insert method
    sub _rmVid {
        my $self = shift;
        my %args = @_;

        $self->_getDBH if (! defined $self->{_dbh});
        my $sth= $self->{_dbh}->prepare( $sql{rmvid} );

        $sth->execute( $args{filename});

    }

    # Generic get data method
    sub _getQueryData {
        my $self = shift;
        my %args = @_;

        croak "Must supply statement" unless (defined $args{sql});

        $self->_getDBH if (! defined $self->{_dbh});

        my $sth= $self->{_dbh}->prepare( $sql{$args{sql}} );
        $sth->execute;

        my @data;
        while (my $retstat = $sth->fetchrow) {      # fetch the rows into a hash reference
            push @data, $retstat;
        }

        return \@data;

    }

    # a call to file::find to find out what files we need.
    sub _getFSVideo {
        my $self = shift;
        my %args = @_;

        croak "Must supply path!" unless (defined $args{path});
        croak "Must supply types!" unless (defined $args{vidtypes});
    
        find( {wanted => sub{ $self->_wanted($args{vidtypes}, $File::Find::name) } }, ${$args{path}}[0]);

        return \@FSfiles;

    }

    # We determine what files we want from the filesystem here.
    sub _wanted {
        my $self = shift;

        my $types = shift;
        my $name  = shift;
  
        # Iterate thru' the allowed file types here 
        foreach (@$types) {
            if (lc(substr($name, -3)) eq lc($_) )  {
                push @FSfiles, $name;
                last;
            }
        }

    }


}

sub new {
    my $pkg = shift;

    my $self = bless { }, $pkg;
    $self->{config}=\%items;

    return $self;
}

sub sync {
    my $self = shift;
    my %args = @_;

    #We build a has for speedy lookups!
    print "Fetching videos from mythvideo..\n" if (defined $args{debug});
    my $mythvids = $self->_getQueryData(sql  => 'mythvids');
    my %mythvideo;
    foreach (@$mythvids) {
        $mythvideo{$_}=1;
    }

    # Go grab the video types from myth
    print "Fetching video types..\n" if (defined $args{debug});
    my $vidtypes = $self->_getQueryData(sql  => 'vidtypes');

    # Grab the location where the video are.
    print "Fetching video location..\n" if (defined $args{debug});
    my $vidloc   = $self->_getQueryData(sql  => 'vidloc'  );

    # Go and get all the video files from the filesystem (FS).
    print "Fetching videos from the filesystem..\n" if (defined $args{debug});
    my $fsvideo = $self->_getFSVideo(path => $vidloc, vidtypes => $vidtypes);
    my %fsvid;
    foreach (@$fsvideo) {
        $fsvid{$_}=1;
    }

    # if fsvideo = mythvids we still want to do the syc as there still maybe
    # video's that are not sync'd, ie this is just a net figure (just an interesting stat)
    # not acutal file difference.
    print "\nFound ".scalar(@$fsvideo)." video files on the FS\n" if (defined $args{debug});
    print "Found ".scalar(@$mythvids)." video files in Mythtv\n" if (defined $args{debug});

    # Perform the check and do the sync operation, ie
    # find the videos that are not in myth and put them there 
    print "\nPerforming filesystem to database syncronisation..\n" if (defined $args{debug});
    my $i = 0;
    foreach (@$fsvideo) {
        if (!defined $mythvideo{$_}){
            my @title = split('/', $_);
            print "\tadding: $_ ". substr($title[$#title], 0, (length($title[$#title])-4))."\n" if (defined $args{debug});
            $i++;
            $self->_addVid(title => substr($title[$#title], 0, (length($title[$#title])-4)), filename => $_);

        }
    }
    print "Filesystem to database syncronisation complete, added $i videos..\n\n" if (defined $args{debug});
    

    # Perform the check and do the sync operation, ie
    # find the videos that are not in myth and put them there
    print "Performing database to filesystem syncronisation..\n" if (defined $args{debug});
    $i=0;
    foreach (@$mythvids) {
        if (!defined $fsvid{$_}){
            my @title = split('/', $_);
            print "\tremoving: $_ ". substr($title[$#title], 0, (length($title[$#title])-4))."\n" if (defined $args{debug});
            $i++;
            $self->_rmVid(sql => 'rmvid', filename => $_);
        }
    }
    print "Database to filesystem syncronisation complete, removed $i videos..\n\n" if (defined $args{debug});
    print "Mythvideo syncronisation complete.\n"; 
 
}


1
